/**
	Calling a python method from C/C++, and extracting its return value
	http://stackoverflow.com/questions/3286448/calling-a-python-method-from-c-c-and-extracting-its-return-value

	embedding python
	http://stackoverflow.com/questions/8229597/embedding-python

	Copyright (c) 2016, Masatsuyo Takahashi, KEK-PF
 */
#include <iostream>
#include <sstream>
#include <Python.h>
#include "autorg_kek_glue.h"

using std::cout;
using std::endl;
using std::stringstream;

PyObject* autorg_func;
PyObject* args_;

AUTORG_KEK_GLUE_API
int Autorg_Initialize( const char* autorg_lib_dir )
{
	// cout << "Autorg_Initialize enter" << endl;

	Py_Initialize();
	// cout << "Py_Initialize ok" << endl;

	PyRun_SimpleString("import os");
	PyRun_SimpleString("import sys");

	stringstream lib_append_ss;
	lib_append_ss << "sys.path.append( '" << autorg_lib_dir << "' )";
	PyRun_SimpleString(lib_append_ss.str().c_str());

	PyObject* name_ = PyUnicode_FromString( (char*)"CommandMain" );
	PyObject* CommandMain_ = PyImport_Import( name_ );
	autorg_func = PyObject_GetAttrString( CommandMain_, (char*)"autorg_func" );
	// cout << "autorg_func=" << autorg_func << endl;
	args_ = PyTuple_New( 4 );
	// cout << "args_=" << args_ << endl;

	return autorg_func == 0 || args_ == 0;
}

AUTORG_KEK_GLUE_API
int Autorg_Execute(const char* data_file, const char* result_file, const int ATSAS, const int robust )
{
	// cout << "Autorg_Execute: " << data_file << " " << result_file << " " << ATSAS << " " << robust <<  endl;

	
	PyObject* data_file_ = PyUnicode_FromString( data_file );
	PyObject* result_file_ = PyUnicode_FromString( result_file );
	PyObject* ATSAS_ = PyLong_FromLong( ATSAS );
	PyObject* robust_ = PyLong_FromLong( robust );

	PyTuple_SetItem( args_, 0, data_file_ );
	PyTuple_SetItem( args_, 1, result_file_ );
	PyTuple_SetItem( args_, 2, ATSAS_ );
	PyTuple_SetItem( args_, 3, robust_ );

	// cout << "Autorg_Execute: before PyObject_CallObject" << endl;

	PyObject* ret = PyObject_CallObject ( autorg_func, args_ );
	return PyLong_AsLong(ret);
}

AUTORG_KEK_GLUE_API
int Autorg_Finalize()
{
	Py_Finalize();
	return 0;
}